/* \TEX{
   \def\srcfile{runtime.c}
   Accurate timings are necessary to compare the algorithms implemented
   in this package. This is done by the function |runtime()|.
   }
*/
#include <stdio.h>
#if defined(linux) || defined(cygnus)
# include <sys/times.h>
# include <sys/param.h>
# ifndef HZ
#  define HZ CLK_TCK
# endif
#else
# if defined(hp) || defined(i486)
#  include <sys/time.h>
#  include <sys/times.h>
#  if defined(i486)
#    include <sys/param.h>
#  endif 
#  ifndef HZ
#   define HZ CLK_TCK
#  endif
# else
#  if defined(solaris)
#   include <sys/times.h>
#   include <sys/param.h>
#   ifndef HZ
#    include <limits.h>
#    define HZ CLK_TCK
#   endif
#  else
#   if defined(alpha) || defined(sgi)
#    include <sys/types.h>
#    include <sys/times.h>
#    if ! defined(HZ) && defined(alpha)
#     include <time.h>
#     define HZ CLK_TCK
#    endif
#    if ! defined(HZ) && defined(sgi)
#     include <limits.h>
#     define HZ CLK_TCK
#    endif
#   else
#    if defined(i486icc)
#     include <time.h>
#    else
#     include <sys/time.h>
#     include <sys/resource.h>
#    endif
#   endif
#  endif
# endif
#endif

#include "sys.h"

/* \TEX{
   \title{Getting system times.}

   \output{The user time (in $10^{-3}$s) of the process that calls
   this function is returned.}
   }
   */
double runtime ()
{
#if defined(hp) || defined(linux) || defined(solaris) || defined(alpha) || defined(i486) || defined(sgi) || defined(cygnus)
  struct tms t;
  double ret;
  
  times(&t);
  ret=t.tms_utime*((double)1000.)/HZ;
  return(ret);
#else
# if defined(i486icc)
  double time1;
  
  time1 = (double)clock();
  time1 = 1000*time1/CLOCKS_PER_SEC;

  return(time1);
# else  
  struct rusage r;
  struct timeval t;
  double ret;
  
  getrusage(0, &r);
  t = r.ru_utime;

  ret=1000.0* ((double)t.tv_sec) + ((double)t.tv_usec)/1000.0;
  return(ret);  
# endif
#endif
}

