/* \TEX{   
   When the amount of shifting is smaller than |SIZE_BLOC|, the macros
   |ZBNShiftLeft| and |ZBNShiftRight| are the quickest way to shift
   BigNum. Otherwise, you can use the routines |ZBNAnyShiftLeft| and
   |ZBNAnyShiftRight|.
   }
   */
#include <stdio.h>
#include "zbn.h"

/* \TEX{
   \title{Shifting left}
   \input{Two allocated |BigNum|s, a pointer on a length, and an integer |nnn|.}
   \side{$|r| = |n| << |nnn|$}
   \note{The size of |r| must be at least $|nl|+|divSizeBloc(nnn)|$, and such that the result of shift can be stored. Hence, an allocated size of $|nl|+|divSizeBloc(nnn)|+1$ is always sufficient. The |BigNum| |n| can be part of |r|. The length of |r| is stored in |*p_rl|. The pointer |p_rl| can be the address of the external value of |nl|.}
   }
   */
void ZBNAnyShiftLeft(r, p_rl, n, nl, nnn)
     BigNum r,n;
     BigNumLength *p_rl,nl;
     unsigned int nnn;
{
  BigNumDigit out;
  BigNumLength ds=0;

  if(nnn >= SIZE_BLOC)
    {    
      ds=divSizeBloc(nnn);
      ZBNAssign(r+ds,n,nl);
      ZBNSetToZero(r,ds);
      r += ds;
      nnn = modSizeBloc(nnn);
    }
  else    
    ZBNAssign(r,n,nl);
 
  out=ZBNShiftLeft(r,nl,nnn);
  if(out) {
    ZBNDigit(r,nl)=out;
    ds++;
  }
  *p_rl = nl+ ds;
}

/* \TEX{
   \title{Shifting right.}
   \input{An allocated |BigNum| |n| of |nl| |BigNumDigits| and an integer |nnn|.}
   \side{$|n| >>= |nnn|$}
   \note{Low degree bits are lost}
   }
   */
void ZBNAnyShiftRight(n,nl,nnn)
     BigNum n;
     BigNumLength nl;
     unsigned int nnn;
{
  if(nnn >= SIZE_BLOC)
    {
      BigNumLength ds;

      ds=divSizeBloc(nnn);
      nl -= ds;
      ZBNAssign(n,n+ds,nl);
      ZBNSetToZero(n+nl,ds);
    }

  ZBNShiftRight(n,nl,modSizeBloc(nnn));
}


