#include <stdio.h>
#include "zbn.h"
#include "zbn_weight_data.h"

#if 0
const BigNumDigit ___Exp2[64] = {
  1uL,
  2uL,
  4uL,
  8uL,
  16uL,
  32uL,
  64uL,
  128uL,
  256uL,
  512uL,
  1024uL,
  2048uL,
  4096uL,
  8192uL,
  16384uL,
  32768uL,
  65536uL,  
  131072uL,
  262144uL,
  524288uL,
  1048576uL,
  2097152uL,
  4194304uL,
  8388608uL,
  16777216uL,
  33554432uL,
  67108864uL,
  134217728uL,
  268435456uL,
  536870912uL,
  1073741824uL,
  2147483648uL,
  4294967296uL,
  8589934592uL,
  17179869184uL,
  34359738368uL,
  68719476736uL,
  137438953472uL,
  274877906944uL,
  549755813888uL,
  1099511627776uL,
  2199023255552uL,
  4398046511104uL,
  8796093022208uL,
  17592186044416uL,
  35184372088832uL,
  70368744177664uL,
  140737488355328uL,
  281474976710656uL,
  562949953421312uL,
  1125899906842624uL,
  2251799813685248uL,
  4503599627370496uL,
  9007199254740992uL,
  18014398509481984uL,
  36028797018963968uL,
  72057594037927936uL,
  144115188075855872uL,
  288230376151711744uL,
  576460752303423488uL,
  1152921504606846976uL,
  2305843009213693952uL,
  4611686018427387904uL,
  9223372036854775808uL
};
#endif

#if LOW_MEMORY
/* \______TEX{
   \title{Weight of a |BigNumDigit|.}
   \input{A BigNum |n|.}
   \output{The number of 1 in the binary representation of |*n| or |ZENERR| if
   an error occored.}
   \side{The first time this function is called, precomputations are
   done. To disallocate them, call this function with |n|=NULL.}
   \note{This function is only available when |LOW_MEMORY| is activated}
   }
   */

unsigned int ZBNWeightDigit(n)
     BigNum n;
{
  static unsigned char *__ZEN_WEIGHT = NULL;
  int w, i, bloc;
  BigNum p;
  BigNumDigit q;

  if (!n) {
    if (__ZEN_WEIGHT) {
      ZENFree(__ZEN_WEIGHT);
      __ZEN_WEIGHT = NULL;
    }
    return(0);
  }
  else {
    if (!__ZEN_WEIGHT) {
      
      if (!(__ZEN_WEIGHT = (unsigned char *)ZENMalloc(BlocExp2(sizeof(short)*BN_BYTE_SIZE)*sizeof(char)))) {
	ZENSetError(ZEN_LIB_ZBN, ZEN_WEIGHT, ZEN_ALLOC);
	return(ZERR);
      }
      
      for(i = 0; i < BlocExp2(sizeof(short)*BN_BYTE_SIZE); i++) {
	bloc = i;
	__ZEN_WEIGHT[i] = 0;
	while(bloc) {
	  bloc &= (bloc-1);
	  __ZEN_WEIGHT[i]++;
	}
      }
    } 
  }
  
  return(_ZBNWeightDigitFast(*n));
}
#else  /* !LOW_MEMORY */

/* \TEX{
   \title{Weight of a |BigNumDigit|}
   \input{A |BigNumDigit| |n|.}
   \output{The number of 1 in the binary representation of |n|.}
   }
   */

unsigned int ZBNWeightDigit(n)
     BigNumDigit n;
{
  return(_ZBNWeightDigitFast(n));
}
#endif

/* \TEX{
   \title{Weight of a BigNum}
   \input{An allocated |BigNum| |n| of size |nl|.}
   \output{The number of 1 in the binary representation of |n| or |ZENERR| if
   an error occored.}
   \side{This function uses |ZBNWeightDigit|.}
   }
   */
int ZBNWeight(n, nl)
     BigNum n;
     BigNumLength nl;
{
  int w;
#if 0
  for(w=0;nl--;n++)
    w+=ZBNWeightDigitFast(*n);
#else
  switch(sizeof(BigNumDigit))
    {
    case sizeof(short):
      for(w=0;nl--;)
	w+=__ZEN_WEIGHT[*(n++)];
      break;
    case (sizeof(short)<<1):
      {
# if 1
	unsigned short *pt;

	pt=(unsigned short *)n;      
	for(w=0;nl--;)
	  {
	    w+=__ZEN_WEIGHT[*(pt++)];
	    w+=__ZEN_WEIGHT[*(pt++)];
	  }
# else
	for(w=0;nl--;n++)
	  w+=((__ZEN_WEIGHT[(*n)&65535]) |((__ZEN_WEIGHT[(*n)>>16])<<16));
	return((w&65535)+(w>>16));
# endif
      }
    break;
    case (sizeof(short)<<2):
      {
	unsigned short *pt;
	
	pt=(unsigned short *)n;
	for(w=0;nl--;)
	  {
	    w+=__ZEN_WEIGHT[*(pt++)];
	    w+=__ZEN_WEIGHT[*(pt++)];
	    w+=__ZEN_WEIGHT[*(pt++)];
	    w+=__ZEN_WEIGHT[*(pt++)];
	  }
      }
    break;
    default:  
      {
	int i;
	unsigned short *pt;
	
	pt=(unsigned short *)n;
	for(w=0;nl--;)
	  for(i=sizeof(BigNumDigit)/sizeof(unsigned short);i--;)
	    w+=__ZEN_WEIGHT[*(pt++)];
      }
    }
#endif
  return(w);
}



