#include "zbn.h"

#define LEHMER 1

/*
   \TEX{
   Some misceallanous functions needed to interface some routines of BigNum 
   }
   */

int ZBNEea(g, p_gl, a, p_al, n, nl, m, ml)
     BigNum g, a, n, m;
     BigNumLength *p_gl, *p_al, nl, ml;
{
  int ret;

#if LEHMER
  ret = BnEeaLehmer(g, p_gl, a, p_al, n, nl, m, ml);
  if (ret == GCD_ERROR) {
    ZENSetError(ZEN_LIB_ZBZ, ZEN_EEALEHMER, ZEN_ALLOC);
  }
#else
  ret = BnEeaPlain(g, p_gl, a, p_al, n, nl, m, ml);
  if (ret == GCD_ERROR) {
    ZENSetError(ZEN_LIB_ZBZ, ZEN_EEAPLAIN, ZEN_ALLOC);
  }
#endif
  return(ret);
}

/*
   \TEX[
\title{Multiplication by a digit}
   \input{Two |BigNum|s of length |nl| and a digit.}
   \output{The most significant digit of $|(m,ml)| \times |d|$.}
   \side{|(n,ml)| is filled with the |ml| least significant digits of $|(m,ml)| \times |d|$.}
   \note{|n| must be distinct from |m|. }
   }
   */
BigNumDigit ZBNMultDigit(n,m,ml,d)
     BigNum n,m;
     BigNumLength ml;
     BigNumDigit d;
{
  BigNumDigit ret[2];
  
  ZBNSetToZero(n,ml);
  ZBNMultiplyDigit(n,ml,m,ml-1,d);
  ret[0]=ZBNDigit(n,ml-1);
  ret[1]=0;
  ZBNMultiplyDigit(ret,2,m+ml-1,1,d);
  ZBNDigit(n,ml-1)=ret[0];
  return(ret[1]);
}

/*
   \TEX[
   \title{Multiplication}
   \input{Three |BigNum|s and their lengths}
   \side{$|(n,nl)| \leftarrow |(a,al)| \times |(b,bl)|$.}
   \note{One must have $|nl| \ge |al|+|bl|$, |al| and |bl| non zeros.}
   } 
   */
void ZBNMult(n,a,al,b,bl)
     BigNum n,a,b;
     BigNumLength al,bl;
{
  ZBNSetToZero(n,al+bl);
  ZBNMultiply(n,al+bl,a,al,b,bl);
}
/*
   \TEX[
   \title{Squaring}
   \input{Two |BigNum|s and a length}
   \side{$|(n,nl)| \leftarrow |(a,al)|^2 $.}
   \note{One must have |n| distinct from |a|, $|nl| \ge 2|al|$, and $|al| > 0$.}
   } 
   */
void ZBNSqu(n,a,al)
     BigNum n,a;
     BigNumLength al;
{
  ZBNSetToZero(n,al<<1);
  ZBNSquare(n,al<<1,a,al);
}

/*
   \TEX{
   \title{Comparison of bignums}
   \input{Two bignums}
   \output{\begin{itemize}
   \item A positive integer if $|m| > |n|$;
   \item zero if $|m| = |n|$;
   \item A negative integer if $|m| < |n|$.\end{itemize}}
   \note{One must have $|m[ml-1]|\neq 0$ and $|n[nl-1]|\neq 0$.}
   } 
   */

int ZBNCompareLazy (mm, ml, nn, nl)
     BigNum	 	mm, nn;
     register BigNumLength 	ml, nl;
{
  register int result = BN_EQ;
    
  if (ml != nl)
    return (ml > nl ? BN_GT : BN_LT);
  
  while (result == BN_EQ && ml-- > 0)
    result = BnnCompareDigits (*(mm+ml), *(nn+ml));
  
  return (result);
}
