#include <stdio.h>
#include "zbn.h"

/* \TEX{
   Usefull functions not defined in GMP.
   }
   */
BigNumLength ZBNNumDigits(n, nl)
     BigNum n;
     BigNumLength nl;
{
  while (nl > 0) {
    if (ZBNDigit(n,nl-1))
      break;
    nl--;
  }
  return(nl ? nl : 1);
}

/**********************************************************************/
int ZBNIsZero(n, nl)
     BigNum n;
     BigNumLength nl;
{
  for(;nl--;)
    if(ZBNDigit(n,nl))
      return(0);
  return(1);
}


/**********************************************************************/
void ZBNComplement(n,  nl) 
     BigNum n;
     BigNumLength nl;
{
  for(;nl--;) 
    ZBNDigit(n,nl) = ~ZBNDigit(n,nl);
}

/**********************************************************************/
void ZBNAssign(n, m, ml)
     BigNum n, m;
     BigNumLength ml;
{
  int i;

  if (n < m) {
    for(i = 0; i < ml; i++) {
      ZBNDigit(n,i) = ZBNDigit(m,i);
    }
  }
  else {
    for(i = ml-1; i >= 0; i--) {
      ZBNDigit(n,i) = ZBNDigit(m,i);
    }
  }
}

/**********************************************************************/
int ZBNCompare(n, nl, m, ml)
     BigNum n; BigNumLength nl;
     BigNum m; BigNumLength ml;
{
  while(nl < ml) {
    if (ZBNDigit(m,ml-1)) {
      return(BN_LT);
    }
    ml--;
  }
  while(ml < nl) {
    if (ZBNDigit(n,nl-1)) {
      return(BN_GT);
    }
    nl--;
  }
  return(ml ? zmpn_cmp(n, m, ml) : BN_EQ);
}

int ZBNCompareLazy (mm, ml, nn, nl)
     BigNum	 	mm, nn;
     BigNumLength 	ml, nl;
{
  if (ml != nl)
    return (ml > nl ? BN_GT : BN_LT);
  
  return(ml ? zmpn_cmp(mm, nn, ml) : BN_EQ);
}
/**********************************************************************/

void ZBNSetRandom(n, nl)
     BigNum n;
     BigNumLength nl;
{
  if (!nl) return;

  nl--;
  do{
     ZBNSetDigit(n+nl,zbnrandom());
  } while(!(*(n+nl)));
  while(nl-- > 0) ZBNSetDigit(n+nl,zbnrandom());
}

/**********************************************************************/
BigNumDigit _ZBNMultiplyPlain(p, pl, m, ml, n, nl)
     BigNum p; BigNumLength pl;
     BigNum m; BigNumLength ml;
     BigNum n; BigNumLength nl;
{
  BigNumDigit carry;
  
  carry = 0;
  while (nl-- > 0) {
    carry += ZBNMultiplyDigit(p, pl, m, ml, *n);
    p++; n++; pl--;
  }
  return(carry);
}

/**********************************************************************/
BigNumDigit _ZBNSquarePlain(p, pl, n, nl)
     BigNum p; BigNumLength pl;
     BigNum n; BigNumLength nl;
{
  BigNumDigit carry;
  BigNumLength ml;
  BigNum m;

  m = n; ml = nl; carry = 0;
  while (nl-- > 0) {
    carry += ZBNMultiplyDigit(p, pl, m, ml, *n);
    p++, n++, pl--;
  }
  return(carry);
}

/**********************************************************************/
unsigned int ZBNNumLeadingZeroBitsInDigit(d) 
     BigNumDigit d;
{
  unsigned int p = 0;
  BigNumDigit mask;

  if (!d) {
    return (SIZE_BLOC);
  }
  mask=BlocExp2(SIZE_BLOC-1);
  
  while ((d & mask) == 0) {
    p++;
    mask >>= 1;
  }
  return (p);
}

/**********************************************************************/
void ZBNDivide(n, nl, d, dl)
     BigNum n, d;
     BigNumLength nl, dl;
{
  unsigned int shift;

  shift=ZBNNumLeadingZeroBitsInDigit(d[dl-1]);
  ZBNShiftLeft(d,dl,shift);
  ZBNShiftLeft(n,nl,shift);
  zmpn_divrem(n+dl, 0, n, nl, d, dl);
  ZBNShiftRight(d, dl, shift);
  ZBNShiftRight(n, dl, shift);
}
/**********************************************************************/
