/*-*-LaTeX-*-
  This is the documentation file of the whole ZEN library. There is
  no C code in this file, only LaTeX commands.


   \TEX{
\input{index}

\def\SubTitle{User's manual}
\input{title}
\thispagestyle{headings}

\tableofcontents  

\Chapter{Introduction}
\input{intro}

\Chapter{Beginning with \zen}
\input{maths}

\Chapter{Understanding \zen}
\input{user}

\Chapter{Enumerating the functions of \zen}
\input{zen}

\Chapter{Implementation principles of |ZEN|.}
\Section{How to write a new arithmetic}
\input{implement}

\Section{Sub-directories of |ZEN|}
\input{dir}


\Section{Generic functions}
  \Subsection{Ring initialization}
  \input{zerdoc}

  \Subsection{Extensions}
  \input{zextdoc}

  \Subsection{General functions}
  \input{zegdoc}

  \Subsection{Default functions}
  \input{zeddoc}

  \Section{Arithmetics}
  \Subsection{Modular rings}
  \input{zepdoc}

  \Subsection{Modular rings with small modulus}
  \input{zepsdoc}

  \Subsection{$\GF{2}$ case}
  \input{ze2doc}

  \Subsection{Rationals}
  \input{zefdoc}

  \Subsection{Clones}
  The purpose of clones is to perform when it is possible some precomputations in order to speed up arithmetic. These precomputations are made upon initialization and can take some time.
 Therefore, cloning can be efficient only if a lot of operations are performed.
 
  \Subsubsection{Tabulated clone}
  \input{zetabdoc}

  \Subsubsection{Logarithm clone}
  \input{zelogdoc}

  \Subsubsection{Use of chinese remainder theorem}
  \input{zecdoc}

\Section{Testing the library}
When adding some new arithmetics to |ZEN|, it is necessary to test these new features. This is done using the standard |zentest| program.

\input{zentest}


\appendix
\Chapter{Big integers functions}
\label{zbn}
  \input{zbn}

\Chapter{Installing \zen}
     \Section{The principle}
     You probably got the |ZEN| package via anonymous ftp in a file
     probably named something like |ZEN.x.y.tgz|. What you have to do
     is:
     \begin{enumerate}
     \item To decompress |ZEN.x.y.tgz| with\\
       $>$| gzip -d $<$ ZEN.x.y.tgz |$\vert$| tar xvf -|.
     \item To go in the |ZEN| directory with\\
       $>$| cd ZEN.x.y|.
     \item \label{archdef} To obtain Makefiles from imakefiles with\\
       $>$| zenmkmf.sh|.
     \item \label{compilation} To compile with\\
       $>$| make|.
     \item \label{documentation} To get the documentation with\\
       $>$| make doc|.
     \end{enumerate}

     At the end of step \ref{compilation}, a library file |libzen.a| is
     compiled in |lib/arch| where |arch| depends on your system and the
     file |zen.h| is in |include|. 
     Moreover, test executables are in |bin/arch|.
     
     At the end of step \ref{documentation}, the user manual is in
     the file |doc/dvi/doc.dvi|. Moreover, you can possibly get the
     ``advanced user manual'' in the file |doc/dvi/advanced.dvi| after
     |make adoc|.
     
     Figure~\ref{fig:archi} enumerates the currently tested
     architectures.  Usually, the operation sequence of
     figure~\ref{fig:compil} will produce the library without problems.
     
     \begin{figure}
       \begin{center}
         \begin{tabular}
           {cccc}
%           \hline
           Vendor & Processor & Operating system & Compilers \\
           \hline
%          \hline
           SUN    &  sparc    & SunOS            & cc,gcc \\
%           \cline{3-4}
                  &           & Solaris          & cc,gcc \\
           \hline
           DEC    &  vax      & Ultrix           & cc,gcc \\
%           \cline{3-4}
                  &  mips     & Ultrix           & cc, gcc \\
%           \cline{2-4}
                  &  alpha    & OSF              & cc, gcc \\
           \hline
           HP     &  PA-RISC  & HP-UX            & cc, gcc \\
           \hline
           IBM    &  RS6000   & AIX              & cc, gcc \\
           \hline
           PC     &  486      & Linux            & gcc \\
%           \cline{3-4}
                  &           & NetBsd           & gcc \\
%           \cline{3-4}
                  &           & FreeBsd          & gcc \\
%           \cline{3-4}
%                  &           & OS/2             & gcc \\
%           \cline{3-4}
                  &           & Microsoft Windows (Cygnus Beta 20)             & gcc \\
           \hline
         \end{tabular}
       \end{center}
       \caption{Portability of \zen}
       \label{fig:archi}
     \end{figure}
     
     \Section{Configuring the compiler options}
     \input{specif}
     
     \begin{figure}
       \begin{center}
         \begin{pspicture*}(-2.5,-15)(15,0)
           \rput(2,-1){\sc Operations sequence}
           \rput(9,-1){\sc Shell command}
           
           \psframe[shadow=true](-2,-3)(6,-2)
           \rput(2,-2.5){Local parameters configuration}
           \psframe(7,-3)(11,-2)
           \rput(9,-2.5){\begin{tabular}{c}
               File editing of \ZENF{specif.def} \\
               \ZENF{imake -Iconfig} \end{tabular}}
           
           \psline{->}(2,-3)(2,-4)
           
           \psframe[shadow=true](-2,-5)(6,-4)
           \rput(2,-4.5){Main file computation}
           \psframe(7,-5)(11,-4)
           \rput(9,-4.5){\ZENF{make Makefile}}
           
           \psline{->}(2,-5)(2,-6)

           \psframe[shadow=true](-2,-7)(6,-6)
           \rput(2,-6.5){Sub-Makefiles computation}
           \psframe(7,-7)(11,-6)
           \rput(9,-6.5){\ZENF{make Makefiles}}
           
           \psline{->}(2,-7)(2,-8)
           
           \psframe[shadow=true](-2,-9)(6,-8)
           \rput(2,-8.5){Library compilation}
           \psframe(7,-9)(11,-8)
           \rput(9,-8.5){\ZENF{make}}
           
           \psline[linestyle=dashed]{->}(2,-9)(2,-10)
           
           \psframe[shadow=true](-2,-11)(6,-10)
           \rput(2,-10.5){Cleaning unnecessary files}
           \psframe(7,-11)(11,-10)
           \rput(9,-10.5){\ZENF{make clean}}
           
           \psline[linestyle=dashed]{->}(2,-11)(2,-12)
           
           \psframe[shadow=true](-2,-13)(6,-12)
           \rput(2,-12.5){Removing all compiled files including library}
           \psframe(7,-13)(11,-12)
           \rput(9,-12.5){\ZENF{make tidy}}
         \end{pspicture*}
         
         
       \end{center}
       \caption{Compilation of the library.}
       \label{fig:compil}
     \end{figure}

     \Section{Known problems}

     We are aware of some problems in installing the library. How to
     fix these problems can be found in the file |bugs.txt| on ftp
     servers together with the package.

See also the note~\ref{zbncompil}, page~\pageref{zbncompil}, if you have memory problems on your computer.      

\Section{Customized memory allocation functions}
     \input{memalloc}

\Chapter{Programming environnement}
     \input{prgenv}





\input{bib}
}
*/
