/* \TEX{
      Following a GMP mechanism, we allows customized allocation functions.
      }
*/
#include <stdio.h>
#include <stdlib.h>
#include "sys.h"

#if 0
/*
   \TEX{
   \title{Allocating memory}
   \input{The size of memory to allocate}
   \output{A pointer on the allocated array or |NULL| if an error occured.}
   \note{This macro is used in the whole |ZEN| and |ZENFACT| libraries when memory allocation is needed. By default, a call to the native |malloc| function is performed. This can be overwritten by |ZENSetMemoryFunctions|.}
   } 
   */
void * ZENMalloc(size)
     size_t size;{}
#endif
void * (*ZENMallocPtr) ___((size_t))  = malloc;

#if 0
/*
   \TEX{
   \title{Reallocating memory}
   \input{The pointer to a previously allocated memory array, obtained by |ZENMalloc| , and the new size needed.}
   \output{A pointer on the newly allocated array, or |NULL| if an error occured.}
   \side{The values stored in the first array designed by |oldptr| are copied in the new array.}
   \note{This macro is used in the whole |ZEN| and |ZENFACT| libraries when memory reallocation is needed. By default, a call to the native |realloc| function is performed. This can be overwritten by |ZENSetMemoryFunctions|.}
   } 
   */
void * ZENRealloc(oldptr,newsize)
     void * oldptr;
     size_t newsize;{}
#endif
void * (*ZENReallocPtr) ___((void *, size_t))  = realloc;
#if 0
/*
   \TEX{
   \title{Freeing memory}
   \input{A pointer to a previously allocated memory array, obtained by |ZENMalloc|.}
   \side{The memory pointed to by |ptr| is freed.}
   \note{This macro is used in the whole |ZEN| and |ZENFACT| libraries when freeing memory is needed. By default, a call to the native |free| function is performed. This can be overwritten by |ZENSetMemoryFunctions|.}
   } 
   */
void ZENFree(ptr)
     void * ptr; {}
#endif
void (*ZENFreePtr) ___((void *))  = free;


#ifndef HasBigNum
void *magmp(s)
     size_t s;
{
  return(ZENMalloc(s));
}
void *regmp(p,o,n)
     void *p;
     size_t o,n;
{
  return(ZENRealloc(p,n));
}
void frgmp(p,o)
     void *p;
     size_t o;
{
  ZENFree(p);
}
#endif

/*
   \TEX{
   \title{Setting custom memory allocation functions}
   \input{The three memory allocation functions performing |malloc|, |realloc|, and |free|.}
   \side{The internal memory allocation functions are customized}
   \note{By default, the standard |malloc|, |realloc|, and |free| functions are used. If |GMP| is used, a call to |mp_set_memory_functions| is also done accordingly. This function should be called at the very beginning of a program.}
   } 
   */
void ZENSetMemoryFunctions(ma,re,fr)
     void * (* ma) ___((size_t));
     void * (* re) ___((void *, size_t));
     void  (* fr) ___((void *));
{
  if(ma)
    ZENMallocPtr=ma;
  if(re)
    ZENReallocPtr=re;
  if(fr)
    ZENFreePtr=fr;

#ifndef HasBigNum
  mp_set_memory_functions(magmp,regmp,frgmp);
#endif
}
