#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys.h>

/*************************************************************/
/*\TEX{ 
   \def\srcfile{parser.c}
   This program builds LaTeX documentation source from the C files comments.
   A few commands are used to perform clean output using |zen.sty| file.
   These commands are only valid inside a C-comment:
   \begin{itemize}
   \item |\TEX{ZeNTeX}| allows to include LaTeX code in the resulting file as is. It is also intended to enumerate the procedures of the C files.
   To include a procedure in the LaTeX file, there are fields that should be used to describe the function, all but the first of which are optional.
   \begin{enumerate}
   \item |\title| should shortly describe the function. This field is needed but can be set to ---.
   \item |\input| should describe the parameters of the function.
   \item |\output| should describe the output of the function.
   \item |\side| should describe the side effects of the procedure.
   \item |\note| allows to precise some programming features.
   \end{enumerate}
   Besides, as soon as a |\title{}| appears in a |\TEX| comment, the C-header of the following function is included and index entries for |MakeIndex| program are generated.
   This feature is disabled by the -noindex option in the command line.

   \item |\TEX[ZeNTeX}| behaves in the same way, except that short descriptionS of the functions are included.
   \item |\CiteC| makes the C source to be included in |verbatim| mode in the LaTeX code.
   \item |\EndCiteC| ends the citation of C source.
   
   \item Pipes symbols $\mid$ may delimit the programs names and declarations. Within pipes, the underscore symbols |_|  are automatically backslashed ({\sf $\backslash$\_}) the stars |*| transformed in {\sf\$$\backslash$star\$} and the backslash |\| transformed in {\sf\$$\backslash$backslash\$}.
   \end{itemize}

   \begin{note}
   By default, the output format of LaTeX is set to |a4paper|. It is possible to use default legal format by modifying the following compilation directive in file |parser.c|.
   }
   \CiteC */
#define A4PAPER 1		/* \EndCiteC */
/*
   \TEX{
   \end{note}   
   } 
   */



FILE *IN,*OUT;

				/* Concept index */
enum glo1 {			/* first level */
  bignum,
  optimization,
  polynom,
  rings,
  series,
  elements,
  curve,
  matrix,
  nbobjets
};

char *objets[nbobjets] = {	
  "big integers",
  "optimization",
  "polynomials",
  "rings",
  "series",
  "elements",
  "elliptic curves",
  "matrices",
};

enum glo2 {			/* second level */
  point,
  permutation,
  digit,
  modular,
  flag,
  clone,
  coeff,
  prc,
  prime,
  irreduc,
  monic,
  debug,
  nblev2
};

char *objets2[nblev2] = {
  "points",
  "permutation",
  "digit",
  "modular",
  "flags",
  "clones",
  "coefficients",
  "precomputations",
  "prime",
  "irreducible",
  "monic",
  "debugging"
};
  
enum glo3 {			/* third level */
  addition,
  negation,
  weight,
  mult,
  exp,
  inverse,
  sub,
  squ,
  create,
  shift,
  assign,
  compare,
  test,
  freeing,
  io,
  trace,
  convert,
  dup,
  gauss,
  det,
  kernel,
  permute,
  rank,
  mod,
  transpose,
  deg,
  derive,
  divide,
  dot,
  gcd,
  root,
  length,
  val,
  resultant,
  eval,
  flags,
  logarithm,
  doubling,
  factor,
  nblev3
};
  
char *objets3[nblev3] = {
  "addition",
  "negation",
  "weight",
  "multiplication",
  "exponentiation",
  "inverse",
  "subtraction",
  "square",
  "create",
  "shifting",
  "assignation",
  "comparison",
  "test",
  "freeing memory",
  "input/output",
  "trace",
  "conversion",
  "duplication",
  "gaussian elimination",
  "determinant",
  "kernel",
  "permuting",
  "rank",
  "modular reduction",
  "transpose",
  "degree",
  "derivation",
  "division",
  "scalar product",
  "greatest common divisor",
  "root",
  "length",
  "valuation",
  "resultant",
  "evaluation",
  "flags",
  "logarithm",
  "double",
  "factorization"
};


				/* FUNCTION INDEX */
enum ind2 {			/* second level */
  generic,
  tests,
  ze2m,
  ze2,
  defaults,
  zext,
  zeps,
  zep,
  zelog,
  zetab,
  zef,
  zec,
  nbfct
};

char *fctlev2[nbfct] = {
  "generic functions",
  "library tests",
  "extension over $\\GF{2}$",
  "$\\GF{2}$",
  "defaults functions",
  "extensions",
  "small modulus functions",
  "modular case",
  "clones with logarithms",
  "clones with tabulation",
  "rational functions",
  "chinese remainder construction"
};

typedef struct {
  enum glo1 l1;
  enum glo2 l2;
  enum glo3 l3;
} conceptentry;

typedef struct {
  enum glo1 l1;
  enum ind2 l2;
} fctentry;




char *header_proc[]={
  "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n",
  NULL};

char *header[]={
  "\\headerZeN{",
  "\\multicolumn{2}{c}{",
  NULL};

char *bottom_proc[]={
  "}}",
  "\\makeprocedure",
  NULL};
char *bottom_silent_proc[]={
  "}}",
  "\\makesilentprocedure",
  NULL};

char *header_tex[]= {
  "\\ifx\\undefined\\ZeNBloc",
  "\\gdef\\@FirstZeNBloc{}",
  "\\newcommand{\\ZeNBloc}[1]{\\if!\\@FirstZeNBloc!{",
  "    \\gdef\\@FirstZeNBloc{done}",
#if A4PAPER
  "    \\documentclass[a4paper]{book}",
#else
  "    \\documentclass{book}",
#endif
  "    \\usepackage{pstricks}",
  "    \\usepackage{ifthen}",
  "    \\usepackage{zen}",
  NULL};

char *header_tex_index[]= {
  "    \\usepackage{makeidx}",
  NULL};

char *header_tex_french[]= {
  "    \\usepackage[francais]{babel}",
  "    \\usepackage[latin1]{inputenc}",
  "    \\def\\procedurename{Proc\\'edure}",
  "    \\def\\inputname{Entr\\'ee~}",
  "    \\def\\outputname{Sortie~}",
  "    \\def\\sidename{Action~}",
  "    \\def\\notename{Note~}",
  NULL};

char *end_header_tex1[]= {
  "    \\setcounter{secnumdepth}{6}\\setcounter{tocdepth}{6}",
  "    \\makeindex",
  "    \\makeglossary",
  "    \\begin{document}#1",
  NULL};
char *end_header_tex2[]= {
  "  \\end{document}}",
  "  \\else #1 \\fi",
  "  }",
  "\\fi",
  "\\ZeNBloc{",
  NULL};

char *bottom_tex[]= {
  "}",
  NULL};

char *Msgerrparse[]= {
  "Unended \\TEX block",
  "Two \\title for the same function",
  "Unended pipe",
  "Unended C citation",
  NULL};

#define UNENDED_TEX_BLOCK 1
#define TWO_TEX_BLOCKS 2
#define UNENDED_PIPE 3
#define UNENDED_CITE_C 4

#ifdef DEBUG_PARSER
# define zengetc(stream) _fgetc(stream)
#else
# define zengetc(stream) fgetc(stream)
#endif

int _fgetc(stream)
     FILE *stream;
{int c;

 c=fgetc(stream);
 fputc(c,stdout);
 fflush(stdout);
 return(c);
}
/*************************************************************/
int FindNextTex ___((int *));
void Output ___((char **));
int FindToken ___((char *,FILE *));
/*************************************************************/
int NO_INDEX,FRENCH;

int main(argc,argv)
     int argc;			
     char **argv;		
{char **msg,file[1024];
 int i,err,ln,FLAG;
 
 IN=stdin;
 NO_INDEX=FRENCH=0;

 for(i=1;i<argc;i++)
   if (strcmp(argv[i],"-noindex")==0)
     NO_INDEX=1;
   else if (strcmp(argv[i],"-french")==0)
     FRENCH=1;
   else
     strcpy(file,argv[i]);

 
 IN=stdin;
 OUT=stdout;

 
 Output(header_tex);
 if(!NO_INDEX)
   Output(header_tex_index);
 if(FRENCH)
   Output(header_tex_french);
 Output(end_header_tex1);
 fprintf(OUT,"\\input{%s.ing}\n",file);
 fprintf(OUT,"\\input{%s.ind}\n",file);
 Output(end_header_tex2);

 ln=1;

 FLAG=0;
 while(err=FindNextTex(&ln))
   {
     fprintf(OUT,"\n");
     fprintf(stderr,"#### Something wrong in input around line %d\n",ln);
     fprintf(stderr,"%s\n",Msgerrparse[err-1]);
     FLAG=1;
   }

 fprintf(OUT,"\n");
 Output(bottom_tex);
 return(FLAG);
}

/*************************************************************/

#define SIZEINDEX 2048
void OutputIndex(idx)
     char idx[SIZEINDEX];
{
  char *pt;
  conceptentry ce;
  fctentry fe;
  int i,lvl=0;
  

  ce.l1=fe.l1=nbobjets;
  ce.l2=nblev2;
  ce.l3=nblev3;
  fe.l2=nbfct;

#ifndef ZEN_NO_TEST
  if(NO_INDEX)
    {
      fprintf(stderr,"No indexing for %s...\n",idx);
      return;
    }
  fprintf(stderr,"Indexing %s...\n",idx);
#endif
  pt=idx;
  
#define check(level,str,ce1,ce2,ce3,le) \
    if((lvl<=level)&&(strncmp(pt,str,strlen(str))==0)) \
    { pt+=strlen(str); \
      if(ce1>=0) ce.l1=ce1; \
      if(ce2>=0) ce.l2=ce2; \
      if(ce3>=0) ce.l3=ce3; \
      if(ce1>=0) fe.l1=ce1; \
      if(le>=0) fe.l2=le; \
      lvl++; } else

  check(0,"ZBN",bignum,-1,-1,-1);
  check(0,"ZEN",-1,-1,-1,-1);
  check(0,"\\_ZEN",-1,-1,-1,generic);
  check(0,"\\_\\_ZEN",-1,-1,-1,generic);
  check(0,"\\_Ze2m",-1,-1,-1,ze2m);
  check(0,"\\_Ze2",-1,-1,-1,ze2);
  check(0,"\\_\\_Ze2",-1,-1,-1,ze2);
  check(0,"Ze2",-1,-1,-1,ze2);
  check(0,"\\_Zeds",-1,-1,-1,defaults);
  check(0,"Zeds",-1,-1,-1,defaults);
  check(0,"\\_Zed",-1,-1,-1,defaults);
  check(0,"Zed",-1,-1,-1,defaults);
  check(0,"\\_Zext",-1,-1,-1,zext);
  check(0,"Zext",-1,-1,-1,zext);
  check(0,"\\_Zeps",-1,-1,-1,zeps);
  check(0,"Zeps",-1,-1,-1,zeps);
  check(0,"\\_Zep",-1,-1,-1,zep);
  check(0,"Zep",-1,-1,-1,zep);
  check(0,"\\_Zelog",-1,-1,-1,zelog);
  check(0,"Zelog",-1,-1,-1,zelog);
  check(0,"\\_Zetab",-1,-1,-1,zetab);
  check(0,"Zetab",-1,-1,-1,zetab);
  check(0,"\\_Zef",-1,-1,-1,zef);
  check(0,"Zef",-1,-1,-1,zef);

  check(0,"div",bignum,digit,-1,-1);
  check(0,"mod",bignum,digit,-1,-1);
  check(0,"Bloc",bignum,digit,-1,-1);
  check(0,"ZBP",bignum,debug,io,-1);
  check(0,"ZBR",bignum,debug,io,-1);
  check(0,"ZEP",elements,debug,io,-1);
  check(0,"ZER",elements,debug,io,-1);
  check(0,"ZPP",polynom,debug,io,-1);
  check(0,"ZPR",polynom,debug,io,-1);
  check(0,"ZMP",matrix,debug,io,-1);
  check(0,"ZMR",matrix,debug,io,-1);
  check(0,"ZSP",series,debug,io,-1);
  check(0,"ZSR",series,debug,io,-1);
  check(0,"DDF",polynom,irreduc,test,-1);
  check(0,"Berlekamp",polynom,irreduc,test,-1);

  check(1,"SetError",-1,debug,-1,-1);
  check(1,"Error",-1,debug,-1,-1);
  check(1,"ERR",-1,debug,-1,nbfct);
  check(1,"NULL",-1,debug,-1,nbfct);
  check(1,"Elt",elements,-1,-1,-1);
  check(1,"SetMemory",optimization,-1,-1,-1);
  check(1,"Clone",elements,-1,convert,-1);
  check(1,"Poly",polynom,-1,-1,-1);
  check(1,"Vect",matrix,-1,convert,-1);
  check(1,"Mat",matrix,-1,-1,-1);
  check(1,"Permutation",matrix,permutation,-1,-1);
  check(1,"Sr",series,-1,-1,-1);
  check(1,"MEc",curve,-1,-1,-1);
  check(1,"Ec",curve,-1,-1,-1);
  check(1,"Base",-1,-1,create,zep);
  check(1,"Shoop",polynom,-1,-1,-1);
  check(1,"Ext",-1,-1,create,zext);
  check(1,"Two",-1,-1,-1,-1);
  check(1,"GetFrom",-1,-1,io,-1);
  check(1,"Get",-1,-1,assign,-1);
  check(1,"Num",-1,digit,length,-1);
  check(1,"Set",-1,-1,assign,-1);
  check(1,"Long",-1,-1,-1,-1);
  check(1,"Small",-1,-1,-1,-1);
  check(1,"Any",-1,-1,-1,-1);
  check(1,"Mod",-1,modular,-1,-1);

  check(2,"Pt",-1,point,-1,-1);
  check(2,"ZTo",bignum,-1,convert,-1);
  check(2,"To",-1,-1,convert,-1);
  check(2,"Absolute",-1,-1,-1,-1);
  check(2,"Long",-1,-1,-1,-1);
  check(2,"Small",-1,-1,-1,-1);
  check(2,"Col",-1,-1,-1,-1);
  check(2,"Row",-1,-1,-1,-1);
  check(2,"One",-1,-1,-1,-1);
  check(2,"SameType",-1,-1,compare,-1);
  check(2,"OppositeType",-1,-1,compare,-1);
  check(2,"Ring",rings,-1,-1,-1);

  check(3,"Cln",optimization,clone,flags,-1);
  check(3,"Prc",optimization,prc,flags,-1);
  check(3,"Clone",optimization,clone,-1,-1);
  check(3,"ulo",-1,-1,mod,-1);
  check(3,"Log",-1,-1,logarithm,-1);
  check(3,"Sqrt",-1,-1,root,-1);
  check(3,"AddPrc",optimization,prc,-1,-1);
  check(3,"RmPrc",optimization,prc,-1,-1);
  check(3,"Add",-1,-1,addition,-1);
  check(3,"Negate",-1,-1,negation,-1);
  check(3,"Complement",-1,-1,negation,-1);
  check(3,"Multiply",-1,-1,mult,-1);
  check(3,"Mult",-1,-1,mult,-1);
  check(3,"Inverse",-1,-1,inverse,-1);
  check(3,"Subtract",-1,-1,sub,-1);
  check(3,"Double",-1,-1,doubling,-1);
  check(3,"SquareRoot",-1,-1,root,-1);
  check(3,"Squ",-1,-1,squ,-1);
  check(3,"Alloc",-1,-1,create,-1);
  check(3,"Create",-1,-1,create,-1);
  check(3,"Chinese",-1,-1,create,zec);
  check(3,"Set",-1,-1,assign,-1);
  check(3,"Next",-1,-1,assign,-1);
  check(3,"Weight",-1,-1,weight,-1);
  check(3,"Unset",-1,-1,assign,-1);
  check(3,"Are",-1,-1,compare,-1);
  check(3,"Assign",-1,-1,assign,-1);
  check(3,"Exp",-1,-1,exp,-1);
  check(3,"Free",-1,-1,freeing,-1);
  check(3,"GetFrom",-1,-1,io,-1);
  check(3,"Get",-1,-1,assign,-1);
  check(3,"Print",-1,-1,io,-1);
  check(3,"Put",-1,-1,io,-1);
  check(3,"Read",-1,-1,io,-1);
  check(3,"Trace",-1,-1,trace,-1);
  check(3,"Init",-1,-1,-1,-1);
  check(3,"Convert",-1,-1,convert,-1);
  check(3,"Copy",-1,-1,dup,-1);
  check(3,"Gauss",-1,-1,gauss,-1);
  check(3,"Det",-1,-1,det,-1);
  check(3,"Kernel",-1,-1,kernel,-1);
  check(3,"Permute",-1,-1,permute,-1);
  check(3,"Rank",-1,-1,rank,-1);
  check(3,"Transpose",-1,-1,transpose,-1);
  check(3,"Karatsuba",-1,-1,-1,-1);
  check(3,"Winograd",-1,-1,mult,-1);
  check(3,"Derive",-1,-1,derive,-1);
  check(3,"Normalize",-1,-1,-1,-1);
  check(3,"Divide",-1,-1,divide,-1);
  check(3,"Dot",-1,-1,dot,-1);
  check(3,"ExtGcd",-1,-1,gcd,-1);
  check(3,"Gcd",-1,-1,gcd,-1);
  check(3,"Eea",-1,-1,gcd,-1);
  check(3,"Has",-1,-1,-1,-1);
  check(3,"Lgt",-1,-1,length,-1);
  check(3,"Make",-1,-1,-1,-1);
  check(3,"Update",-1,-1,-1,-1);
  check(3,"Resultant",-1,-1,resultant,-1);
  check(3,"Eval",-1,-1,eval,-1);
  check(3,"Size",-1,-1,length,-1);
  check(3,"From",-1,-1,convert,-1);
  check(3,"To",-1,-1,convert,-1);
  check(3,"Full",-1,-1,-1,-1);
  check(3,"IsNot",-1,-1,test,-1);
  check(3,"IsA",-1,-1,test,-1);
  check(3,"Is",-1,-1,test,-1);
  check(3,"2",-1,-1,convert,-1);

  check(4,"Camion",-1,-1,factor,-1);
  check(4,"ThiongLy",-1,-1,factor,-1);
  check(4,"MillerRabin",-1,prime,test,-1);
  check(4,"Prime",-1,prime,-1,-1);
  check(4,"Monic",-1,monic,-1,-1);
  check(4,"Deg",-1,-1,deg,-1);
  check(4,"Val",-1,-1,val,-1);
  check(4,"Close",-1,-1,freeing,-1);
  check(4,"Shift",-1,-1,shift,-1);
  check(4,"Coeff",-1,coeff,-1,-1);
  check(4,"DRoot",-1,-1,root,-1);
  check(4,"Root",-1,-1,root,-1);
  check(4,"Digit",-1,digit,-1,-1);
  check(4,"Good",-1,-1,-1,-1);
  check(4,"Random",-1,-1,-1,-1);
  check(4,"Smallest",-1,-1,-1,-1);
  check(4,"ED",-1,-1,-1,-1);
  check(4,"DD",-1,-1,-1,-1);
  check(4,"SF",-1,-1,-1,-1);

  check(5,"Fact",-1,-1,factor,-1);
  check(5,"GCD",-1,-1,gcd,-1);
  check(5,"Irreducible",-1,irreduc,-1,-1);

  check(1,"Berlekamp",polynom,irreduc,-1,-1);
  check(1,"C",-1,-1,create,-1);
  check(1,"F",-1,-1,freeing,-1);

#ifdef ZEN_NO_TEST
# define DEBUGINDEX 0
#else
# define DEBUGINDEX 1
#endif  
  fprintf(OUT,"\\glossary{");
  if(ce.l1<nbobjets)
    {
      fprintf(OUT,"%s@{\\Large\\bf %s}",objets[ce.l1],objets[ce.l1]);
      if(ce.l2<nblev2)
	{
	  fprintf(OUT,"!%s@{\\large\\bf %s}",objets2[ce.l2],objets2[ce.l2]);
	  if(ce.l3<nblev3)
	    fprintf(OUT,"!%s",objets3[ce.l3]);
#if DEBUGINDEX
	  else
	    fprintf(OUT,"!{\\sf %s}",idx);
#endif
	}
      else
	{
	  if(ce.l3<nblev3)
	    fprintf(OUT,"!%s@{\\large\\bf %s}",objets3[ce.l3],objets3[ce.l3]);
#if DEBUGINDEX
	  fprintf(OUT,"!{\\sf %s}",idx);
#endif
	}
    }
  else
    {
      if(ce.l2<nblev2)
	{
	  fprintf(OUT,"%s@{\\Large\\bf %s}",objets2[ce.l2],objets2[ce.l2]);
	  if(ce.l3<nblev3)
	    fprintf(OUT,"!%s@{\\large\\bf %s}",objets3[ce.l3],objets3[ce.l3]);
#if DEBUGINDEX
	  fprintf(OUT,"!{\\sf %s}",idx);
#endif
	}
      else
	{
	  if(ce.l3<nblev3)
	    {
	      fprintf(OUT,"%s@{\\Large\\bf %s}",objets3[ce.l3],objets3[ce.l3]);
#if DEBUGINDEX
	      fprintf(OUT,"!{\\sf %s}",idx);	    
#endif
	    }
#if DEBUGINDEX
	  else
	    fprintf(OUT,"{\\sf %s}",idx);
#endif	  
	}
    }
  fprintf(OUT,"}\n");

  fprintf(OUT,"\\index{");
  if(fe.l1<nbobjets)
    {
      fprintf(OUT,"%s@{\\Large\\bf %s}",objets[fe.l1],objets[fe.l1]);
      if(fe.l2<nbfct)
	fprintf(OUT,"!%s@{\\large\\bf %s}",fctlev2[fe.l2],fctlev2[fe.l2]);
      fprintf(OUT,"!");
    }
  else
    {
      if(ce.l2<nbfct)
	{
	  fprintf(OUT,"%s@{\\Large\\bf %s}",objets2[ce.l2],objets2[ce.l2]);
	  if(fe.l2<nbfct)
	    fprintf(OUT,"!%s@{\\large\\bf %s}",fctlev2[fe.l2],fctlev2[fe.l2]);
	  fprintf(OUT,"!");
	}
      else if(fe.l2<nbfct)
	{
	  fprintf(OUT,"%s@{\\Large\\bf %s}",fctlev2[fe.l2],fctlev2[fe.l2]);
	  fprintf(OUT,"!");
	}      
    }  
  fprintf(OUT,"{\\sf %s}",idx);
  fprintf(OUT,"}\n");
}

int FindNextTex(ln)
     int *ln;
{int c;
 char FLAG,HEAD,INDEX,PIPE,HEAD_COMMENTS,CITE_C,SILENT_TEX;
 char memidx[SIZEINDEX],*idx;
 char unsidx[SIZEINDEX],*uns;

#define NOT_COMMENTS 0
#define COMMENTS 1
#define TEX_BLOCK 2

 
#define FORGET 0
#define SWALLOW 1
#define CONTINUE 2
#define FOUND 3
#define JUMP 4

 
 FLAG=NOT_COMMENTS;
 HEAD=INDEX=FORGET;
 PIPE=CITE_C=0;
 HEAD_COMMENTS=0;
 SILENT_TEX=0;
 idx=memidx;
 uns=unsidx;
 
 do
   {
     c=zengetc(IN);

     if(c == 012)
       (*ln) ++;
     
     switch(FLAG)
       {
/*************************/
        case NOT_COMMENTS :
	  if(CITE_C)
	    {
	      switch(c)
		{
		case EOF:
		  fprintf(OUT,"\\E");
		  fprintf(OUT,"ndOfCite }\n");
		  return(0);

		case ' ':
		  fprintf(OUT,"\\hspace*{2mm}");
		  break;

		case '\t':
		  fprintf(OUT,"\\hspace*{16mm}");
		  break;

		case '/':	/* looking for beginning of comments */
		  c=zengetc(IN);
		  if(c == '*')
		    {
		      FLAG=COMMENTS;
		      fprintf(OUT,"}{\\small ");
		    }
		  else
		    fprintf(OUT,"/%c",c);
		break;

		case '~':
		  fprintf(OUT,"\\~{}");
		  break;

		case '^':
		  fprintf(OUT,"\\^{}");
		  break;

		case '\\':
		  fprintf(OUT,"$\\backslash$");
		  break;
		  /* 
		case '-':
		  c=zengetc(IN);
		  if(c == '>')
		    fprintf(OUT,"$\\rightarrow$");		    
		  else
		    fprintf(OUT,"-%c",c);
		  break;
		  */
		case '\n':
		   fputc('\\',OUT);
		   
		case '#':
		case '$':
		case '%':
		case '&':
		case '_':
		case '{':
		case '}':
		  fputc('\\',OUT);
		  
		default:
		  fputc(c,OUT);
		}
	    }
	  else
	    {
	      switch(HEAD)
		{
		case FOUND :		/* if we currently write a header */
		  switch(c)
		    {
		    case '{':	/* this is the end of the header */
		      HEAD=FORGET;
		      if(SILENT_TEX)
			{
			  Output(bottom_silent_proc);
			  SILENT_TEX=0;
			}
		      else
			Output(bottom_proc);
		      if(INDEX)	/* if we haven't yet output the index */
			{
			  *idx=0;
			  idx=memidx;
			  uns=unsidx;			  
			  OutputIndex(memidx);
			  INDEX=FORGET;
			}
		      break;
		    case 012 :
		      if(SILENT_TEX) /* first newline, we stop... */
			{
			  Output(bottom_silent_proc);
			  SILENT_TEX=0;
			  HEAD=FORGET;
			  if(INDEX)	/* if we haven't yet output the index */
			    {
			      *idx=0;
			      idx=memidx;
			      uns=unsidx;			  
			      OutputIndex(memidx);
			      INDEX=FORGET;
			    }
			}
		      else
			{
			  fprintf(OUT,"}\\\\&{\n");
			  if(INDEX==FOUND)	/* if we have found the index */
			    INDEX=CONTINUE;
			}
		      break;
		      
		    case '/' :
		      c=zengetc(IN);
		      if(c == '*')
			{
			  HEAD_COMMENTS=1;
			  fprintf(OUT,"\\makebox[0in][l]{\\rm\\small /*");/**/
			    if(INDEX==FOUND)
			      INDEX=JUMP;
			}
		      else
			{
			  fputc('/',OUT);
			  fputc(c,OUT);
			  if(INDEX==FOUND)
			    {
			      *idx++ ='/';
			      *idx++ = c;
			    }
			}
		      break;
		      
		    case '*' :
		      if(HEAD_COMMENTS)
			{
			  fputc('*',OUT);
			  do
			    {
			      c=zengetc(IN);
			      fputc(c,OUT);
			    } while(c == '*');
			  if(c == '/')
			    {
			      HEAD_COMMENTS=0;
			      fputc('}',OUT);
			      if(INDEX==JUMP)
				INDEX=FOUND;
			    }
			}
		      else
			fputc('*',OUT);
		      break;
			  
		    case '_':
		      fprintf(OUT,"\\_");
		      if(INDEX==FOUND)
			{
			  *idx++ = '\\';
			  *idx++ = '_';
			}
		      break;
		      
		    default:
		      fputc(c,OUT);	/* we write everything */
		      if(INDEX==FOUND)
			*idx++ = c;
		      else if((INDEX==SWALLOW)
			      &&(HEAD_COMMENTS==0)
			      &&(c == ' '))
			{
			  *uns=0;
			  if(strncmp(unsidx+1,"signed",strlen("signed"))) /* sordid hack !
									     I have no time to make that
									     better */
			    INDEX=FOUND;
			  uns=unsidx;
			}
		      else
			*uns++=c;
		    }
		    
		  break;
		  
		default :
		  if(c == '/')		/* looking for beginning of comments */
		    {
		      c=zengetc(IN);
		      if(c == '*')
			{
			  FLAG=COMMENTS;
			  if(HEAD)	/* if this is a comment while
					   waiting for a header, we memorize it */
			    HEAD=CONTINUE;
			}
		    }
		  else if (HEAD)	/* we are looking for a header */
		    if((c != 012)
		       &&(c != '\t')
		       &&(c != ' ')) /* we swallow the white spaces */
		      {
			HEAD = FOUND;
			INDEX=SWALLOW;
			if(c == '_')
			  fprintf(OUT,"\\_");
			else
			  fputc(c,OUT);
		      }
		}
	    }
	  break;
/*************************/
       case COMMENTS :
	 switch(c)
	   {
	   case '\\':		/* looking for a TEX block */
	     c=zengetc(IN);
	     switch(c)
	       {
	       case 'T':
		 if(FindToken("EX",NULL))
		   {
		     FLAG = TEX_BLOCK;
		     c=zengetc(IN);
		     switch(c)
		       {
		       case '[':
			 SILENT_TEX=1;
		       case '{':
			 Output(header_proc);
			 break;
		       default:
			 break;
		       }
		   }
		 break;

	       case 'C':
		 if(FindToken("iteC",NULL))
		   {
		     if(CITE_C)
		       return(UNENDED_CITE_C);
		     
		     c='\n';
		     CITE_C= (char)-1;
		   }
		 break;

	       case 'E':
		 if(CITE_C)
		   {
		     fprintf(OUT,"\\E");
		     if(FindToken("ndCiteC",OUT))
		       {
			 fprintf(OUT,"ndOfCite }\n");
			 CITE_C=0;
		       }
		   }
		 break;

	       default:
		 if(CITE_C)
		   {
		     fputc('\\',OUT);
		     fputc(c,OUT);
		   }
	       }
	     break;
	     
	   case '*':		/* looking for the end of comments */
	     do
	       c=zengetc(IN);
	     while(c == '*');
	     
	     if(c == '/')
	       {
		 FLAG=NOT_COMMENTS;
		 if(HEAD == SWALLOW) /* the first time we are here we
				      output \ZENHeader */
		   Output(header);
		 switch(CITE_C)
		   {
		   case (char)-1:
		     fprintf(OUT,"\n\\\n{\\tt ");
		     CITE_C=1;
		     break;
		   case 1:
		     fprintf(OUT," }{\\tt ");
		     break;
		   }
	       }
	     break;

	   default:
	     if(CITE_C)
	       switch(c)
		 {
		 case '\n':
		   fputc('\\',OUT);
		   fputc(' ',OUT);
		   fputc('\\',OUT);
		   
		 case '#':
		 case '$':
		 case '%':
		 case '&':
		 case '~':
		 case '_':
		 case '^':
		 case '{':
		 case '}':
		   fputc('\\',OUT);
		   
		 default:
		   fputc(c,OUT);
		 }
	   }
	  
	  break;

/*************************/
        default :
	 			/* we are in a TEX Block */
	 switch(c)
	   {
	   case '{' :		/* we count the brackets */
	     if(PIPE)
	       fprintf(OUT,"\\{");
	     else
	       {
		 FLAG++;
		 fputc(c,OUT);
	       }
	     break;
		 
	   case '}' :
	     if(PIPE)
	       fprintf(OUT,"\\}");
	     else
	       {
		 FLAG--;
		 if(FLAG>=TEX_BLOCK)
		   fputc(c,OUT);
		 else if (CITE_C)
		   return(UNENDED_CITE_C);
	       }
	     break;
	     
	   case '|' :
	     if (PIPE)
	       fputc('}',OUT);
	     else
	       fprintf(OUT,"\\mbox{\\sf ");
	     PIPE=!(PIPE);
	     break;
	     
	   case '*' :		/* we check for end of comments and return
				   error in this case */
	     if(PIPE)
	       fprintf(OUT,"\\mbox{$\\star$}");
	     else
	       {
		 fputc('*',OUT);
		 do
		   {
		     c=zengetc(IN);
		     fputc(c,OUT);
		   } while(c == '*');

		 switch(c)
		   {
		   case '{' :		/* we count the brackets */
		     FLAG++;
		     break;
		 
		   case '}' :
		     FLAG--;
		     if((FLAG < TEX_BLOCK)
			&&(CITE_C))
		       return(UNENDED_CITE_C);
		     break;
		     
		   case '/':
		     return(UNENDED_TEX_BLOCK);
		   }
	       }
	     break;

	   case '-':
	     if(PIPE)
	       {
		 c=zengetc(IN);
		 if(c == '>')
		   fprintf(OUT,"$\\rightarrow$");		    
		 else if(c == '|')
		   {
		     PIPE=!(PIPE);
		     fputc('}',OUT);
		   }
		 else
		   fprintf(OUT,"-%c",c);
	       }
	     else
	       fputc('-',OUT);		 
	     break;

	   case '\\' :		/* this is certainly a command */
	     if(PIPE)
	       fprintf(OUT,"\\mbox{$\\backslash$}");
	     else
	       {
		 fputc('\\',OUT);
		 if(FLAG == TEX_BLOCK) /* if we are at root for LaTeX */
		   {
		     c=zengetc(IN);
		     switch(c)
		       {
		       case 't':
			 fputc('t',OUT);
			 if(FindToken("itle{",OUT))
			   {
			     fprintf(OUT,"itleZeN{");
			     FLAG++;
			     if(HEAD)
			       return(TWO_TEX_BLOCKS);
			     HEAD=SWALLOW;
			   }
			 break;
		       case 'i':
			 fputc('i',OUT);
			 if(FindToken("nput{",OUT))
			   {
			     FLAG++;
			     if (HEAD == SWALLOW) {
			       fprintf(OUT,"nputZeN{");
			     }
			     else {
			       fprintf(OUT,"nput{");
			     }
			   }
			 break;
		       case 'o':
			 fputc('o',OUT);
			 if(FindToken("utput{",OUT))
			   {
			     FLAG++;
			     fprintf(OUT,"utputZeN{");
			   }
			 break;
		       case 's':
			 fputc('s',OUT);
			 if(FindToken("ide{",OUT))
			   {
			     FLAG++;
			     fprintf(OUT,"ideZeN{");
			   }
			 break;
		       case 'n':
			 fputc('n',OUT);
			 if(FindToken("ote{",OUT))
			   {
			     FLAG++;
			     fprintf(OUT,"oteZeN{");
			   }
			 break;

		       default:
			 fputc(c,OUT);
		       }
		   }
		 break;
	       case '>':
		 if(PIPE)
		   fprintf(OUT,"\\mbox{$>$}",c);
		 else
		   fputc(c,OUT);
		 break;		 
	       case '_' :
	       case '&' :
		 if(PIPE)
		   fprintf(OUT,"\\%c",c);
		 else
		   fputc(c,OUT);
		 break;
	       default :
		 fputc(c,OUT);
	       }
	   }
       }
     
   } while(c != EOF);

 return(0);
}

/*************************************************************/
int FindToken(tok,out)
     char *tok;
     FILE *out;
{int i,c,l;

 l=strlen(tok);
 if(l==0)
   return(0);
 i=0;

 do
   {
     c=zengetc(IN);
     if(c != tok[i])
       {
	 if(out != NULL)
	   {
	     for(l=0;l<i;l++)
	       fputc(tok[l],out);
	     fputc(c,out);
	   }
	 return(0);
     }
     i++;
   } while(i<l);
 return(1);
}

/*************************************************************/
void Output(msg)
     char *msg[];
{int i;

 if(msg!=NULL)
   for(i=0;
       msg[i]!=NULL;
       i++)
     fprintf(OUT,"%s\n",msg[i]);
}

/*************************************************************/
