/* \TEX{
   \Subsection{Points on elliptic curve creation}
   
   |_ZENEcPtCreate| allocates one point and |_ZENEcPtFree| frees then it.
   }
   */
#include <stdio.h>
#include "zen.h"

/* \TEX{
   \title{Point allocation}
   \input{An elliptic curve |E|.}
   \output{An allocated point.}
   }
   */
ZENEcPt _ZENEcPtCreate(E)
     ZENEc E;
{
  ZENEcPt P;

  if (!(P = ZENEcPtAllocStruct(E))) {
    ZENSetError(ZEN_LIB_ZEG, ZEN_ECPTCREATE, ZEN_ALLOC);
    return(NULL);
  }
  if (ZENEltCreate(ZENEcPtX(P, E), ZENEcRing(E))) {
    ZENFree(P);
    return(NULL);
  }
  if (ZENEltCreate(ZENEcPtY(P, E), ZENEcRing(E))) {
    ZENEltFree(ZENEcPtX(P, E), ZENEcRing(E));
    ZENFree(P);
    return(NULL);
  }
  return(P);
}


/* \TEX{
   \title{Point freeing}
   \input{An allocated point |P| of an elliptic curve |E|.}
   \side{|P| is disallocated.}
   }
   */
void _ZENEcPtFree(P, E)
     ZENEcPt P;
     ZENEc E;
{
  ZENEltFree(ZENEcPtX(P, E), ZENEcRing(E));
  ZENEltFree(ZENEcPtY(P, E), ZENEcRing(E));
  ZENFree(P);
}
