/*
   \TEX{
   \Subsection{Element assignation}
   } 
   */

#include <stdio.h>
#include "zen.h"

/*
   \TEX[
   \title{Converting a |BigNum| to a |ZENElt|}
   \input{A |ZENElt|, a |BigNum| and its length and a |ZENRing|.}
   \output{|ZENERR| if an error occured, 0 otherwise.}
   \side{The big integer is first reduced modulo the characteristic of the field. The result is then assigned to |e| using |ZENZToElt|.}
   } 
   */
int _ZENEltFromBigNum(e,n,nl,R)
     ZENElt *e;
     BigNum n;
     BigNumLength nl;
     ZENRing R;
{
  BigNum rm;
  BigNumLength rml;
  
  rml = ZBNModuloLazy(&rm,n,ZBNNumDigits(n,nl),ZENRingP(R),ZENRingPl(R));
  
  if(!rml) {
    return(ZERR);
  }
  ZENZToElt(*e,rm,rml,R);
  ZBNF(rm);
  return(0);
}

/* \TEX[
   \title{Conversion.}
   \input{A pointer |pa| on a |ZENElt| of a clone ring |Rg|, an element |b| of a ring |R|.}
   \side{|*pa| is set to the modular conversion of |b|.}
   \output{|ZENERR| if an error occured, 0 otherwise.}
   }
   */
int _ZENEltCloneConvert(pa, Rg, b, R)
     ZENElt *pa,b;
     ZENRing Rg,R;
{
  if(ZENEltConvert(ZENRingIbuf(ZENRingOrigin(Rg)),ZENRingOrigin(Rg),b,R))
    return(ZERR);
  ZENEltToClone(*pa,ZENRingIbuf(ZENRingOrigin(Rg)),Rg);
  return(ZEN_HAS_INVERSE);
}
