/*
  \TEX{

   \Subsection{Polynomial creation}
   
   |_ZetabPolyCreate| allocates one |ZENPoly| and |_ZetabPolyFree| frees then it.
   }
   */
#include <stdio.h>
#include "zetab.h"

/* \TEX{
   \title{Creation.}
   \input{A finite ring |Rg| and a degree |deg|.}
   \output{An allocated |ZENPoly| set to 0 with |deg| coefficients allocated. }
   }
   */
ZENPoly _ZetabPolyCreate(deg, Rg)
     int deg;
     ZENRing Rg;
{
  ZENPoly PX;
  int lt;

  
  if (!(PX = ZENPolyAllocStruct(Rg))) {
    ZENSetError(ZEN_LIB_ZETAB, ZEN_POLYCREATE, ZEN_ALLOC);
    return(NULL);
  }
  lt=divSizeBloc((deg+1)*SIZE_CHAR)+1;
  if (!(ZetabPolyCof(PX, Rg) = (unsigned char *)ZENMalloc(lt*sizeof(BigNumDigit)))) {
    ZENFree(PX);
    ZENSetError(ZEN_LIB_ZETAB, ZEN_POLYCREATE, ZEN_ALLOC);
    return(NULL);
  }
  ZetabPolyLgt(PX, Rg) = lt*sizeof(BigNumDigit)/sizeof(unsigned char) - 1;
  return(PX);
}

/* \TEX{
   \title{Free.}
   \input{a |ZENPoly| |PX| of a finite ring |Rg| already allocated with
   |_ZetabPolyCreate|.}
   \side{|PX| is freed.}
   }
   */
void _ZetabPolyFree(PX,Rg)
     ZENPoly PX;
     ZENRing Rg;
{
  ZENFree(ZetabPolyCof(PX, Rg));
  ZENFree(PX);
}
