/* \TEX{
   \Subsection{Polynomial creation}
   
   |_ZextPolyCreate| allocates one |ZENPoly| and |_ZextPolyFree| frees then it.
   }
   */
#include <stdio.h>
#include "zext.h"

/* \TEX{
   \title{Creation.}
   \input{A finite ring |Rg| and a degree |deg|.}
   \output{An allocated |ZENPoly| set to 0 with |deg| coefficients allocated. }
   }
   */
ZENPoly _ZextPolyCreate(deg, Rg)
     int deg;
     ZENRing Rg;
{
  ZENPoly PX;
  int i;

  
  if (!(PX = ZENPolyAllocStruct(Rg))) {
    ZENSetError(ZEN_LIB_ZEXT, ZEN_POLYCREATE, ZEN_ALLOC);
    return(ZNULL);
  }
  if (!(ZextPolyCof(PX, Rg) = (ZENElt *)ZENMalloc(sizeof(ZENElt)*(deg+1)))) {
    ZENFree(PX);
    ZENSetError(ZEN_LIB_ZEXT, ZEN_POLYCREATE, ZEN_ALLOC);
    return(ZNULL);
  }
  for(i = 0; i <= deg; i++) {
    if (ZextEltCreate(ZextPolyGetCof(PX, i, Rg), Rg)) {
      for(i--; i>= 0; i--) {
	ZextEltFree(ZextPolyGetCof(PX, i, Rg), Rg);
      }
      ZENFree(ZextPolyCof(PX, Rg));
      ZENFree(PX);
      return(ZNULL);
    }
  }
  ZextPolyLgt(PX, Rg) = deg;
  return(PX);
}
ZENPoly _ZextPolyCreateLarge(deg, Rg)
     int deg;
     ZENRing Rg;
{
  ZENPoly PX;
  int i;

  
  if (!(PX = ZENPolyAllocStruct(Rg))) {
    ZENSetError(ZEN_LIB_ZEXT, ZEN_POLYCREATE, ZEN_ALLOC);
    return(ZNULL);
  }
  if (!(ZextPolyCof(PX, Rg) = (ZENElt *)ZENMalloc(sizeof(ZENElt)*(deg+1)))) {
    ZENFree(PX);
    ZENSetError(ZEN_LIB_ZEXT, ZEN_POLYCREATE, ZEN_ALLOC);
    return(ZNULL);
  }
  for(i = 0; i <= deg; i++) {
    if (ZextEltCreateLarge(ZextPolyGetCof(PX, i, Rg), Rg)) {
      for(i--; i>= 0; i--) {
	ZextEltFree(ZextPolyGetCof(PX, i, Rg), Rg);
      }
      ZENFree(ZextPolyCof(PX, Rg));
      ZENFree(PX);
      return(ZNULL);
    }
  }
  ZextPolyLgt(PX, Rg) = deg;
  return(PX);
}

/* \TEX{
   \title{Free.}
   \input{a |ZENPoly| |PX| of a finite ring |Rg| already allocated with
   |_ZextPolyCreate|.}
   \side{|PX| is freed.}
   }
   */
void _ZextPolyFree(PX,Rg)
     ZENPoly PX;
     ZENRing Rg;
{
  int i;

  for(i = 0; i <= ZextPolyLgt(PX, Rg); i++) {
    ZextEltFree(ZextPolyGetCof(PX, i, Rg), Rg);
  }
  ZENFree(ZextPolyCof(PX, Rg));
  ZENFree(PX);
}
